@@ -8,13 +8,20 @@
 #ifdef FLECS_JSON
 
 static
-int json_typeinfo_ser_type(
+int flecs_json_typeinfo_ser_type(
     const ecs_world_t *world,
     ecs_entity_t type,
     ecs_strbuf_t *buf);
 
 static
-int json_typeinfo_ser_primitive(
+int flecs_json_typeinfo_ser_type_slice(
+    const ecs_world_t *world,
+    const ecs_meta_op_t *ops,
+    int32_t op_count,
+    ecs_strbuf_t *str);
+
+static
+int flecs_json_typeinfo_ser_primitive(
     ecs_primitive_kind_t kind,
     ecs_strbuf_t *str) 
 {
@@ -59,7 +66,7 @@ int json_typeinfo_ser_primitive(
 }
 
 static
-void json_typeinfo_ser_constants(
+void flecs_json_typeinfo_ser_constants(
     const ecs_world_t *world,
     ecs_entity_t type,
     ecs_strbuf_t *str)
@@ -75,86 +82,28 @@ void json_typeinfo_ser_constants(
 }
 
 static
-void json_typeinfo_ser_enum(
+void flecs_json_typeinfo_ser_enum(
     const ecs_world_t *world,
     ecs_entity_t type,
     ecs_strbuf_t *str)
 {
     ecs_strbuf_list_appendstr(str, "\"enum\"");
-    json_typeinfo_ser_constants(world, type, str);
+    flecs_json_typeinfo_ser_constants(world, type, str);
 }
 
 static
-void json_typeinfo_ser_bitmask(
+void flecs_json_typeinfo_ser_bitmask(
     const ecs_world_t *world,
     ecs_entity_t type,
     ecs_strbuf_t *str)
 {
     ecs_strbuf_list_appendstr(str, "\"bitmask\"");
-    json_typeinfo_ser_constants(world, type, str);
-}
-
-static
-int json_typeinfo_ser_array(
-    const ecs_world_t *world,
-    ecs_entity_t elem_type,
-    int32_t count,
-    ecs_strbuf_t *str)
-{
-    ecs_strbuf_list_appendstr(str, "\"array\"");
-
-    flecs_json_next(str);
-    if (json_typeinfo_ser_type(world, elem_type, str)) {
-        goto error;
-    }
-
-    ecs_strbuf_list_append(str, "%u", count);
-    return 0;
-error:
-    return -1;
-}
-
-static
-int json_typeinfo_ser_array_type(
-    const ecs_world_t *world,
-    ecs_entity_t type,
-    ecs_strbuf_t *str)
-{
-    const EcsArray *arr = ecs_get(world, type, EcsArray);
-    ecs_assert(arr != NULL, ECS_INTERNAL_ERROR, NULL);
-    if (json_typeinfo_ser_array(world, arr->type, arr->count, str)) {
-        goto error;
-    }
-
-    return 0;
-error:
-    return -1;
-}
-
-static
-int json_typeinfo_ser_vector(
-    const ecs_world_t *world,
-    ecs_entity_t type,
-    ecs_strbuf_t *str)
-{
-    const EcsVector *arr = ecs_get(world, type, EcsVector);
-    ecs_assert(arr != NULL, ECS_INTERNAL_ERROR, NULL);
-
-    ecs_strbuf_list_appendstr(str, "\"vector\"");
-
-    flecs_json_next(str);
-    if (json_typeinfo_ser_type(world, arr->type, str)) {
-        goto error;
-    }
-
-    return 0;
-error:
-    return -1;
+    flecs_json_typeinfo_ser_constants(world, type, str);
 }
 
 /* Serialize unit information */
 static
-int json_typeinfo_ser_unit(
+int flecs_json_typeinfo_ser_unit(
     const ecs_world_t *world,
     ecs_strbuf_t *str,
     ecs_entity_t unit) 
@@ -179,7 +128,7 @@ int json_typeinfo_ser_unit(
 }
 
 static
-void json_typeinfo_ser_range(
+void flecs_json_typeinfo_ser_range(
     ecs_strbuf_t *str,
     const char *kind,
     ecs_member_value_range_t *range)
@@ -193,109 +142,63 @@ void json_typeinfo_ser_range(
     flecs_json_array_pop(str);
 }
 
-/* Forward serialization to the different type kinds */
 static
-int json_typeinfo_ser_type_op(
+int flecs_json_typeinfo_ser_scope(
     const ecs_world_t *world,
-    ecs_meta_type_op_t *op, 
-    ecs_strbuf_t *str,
-    const EcsStruct *st) 
+    const ecs_meta_op_t *op,
+    ecs_strbuf_t *str)
 {
-    if (op->kind == EcsOpOpaque) {
-        const EcsOpaque *ct = ecs_get(world, op->type, 
-            EcsOpaque);
-        ecs_assert(ct != NULL, ECS_INTERNAL_ERROR, NULL);
-        return json_typeinfo_ser_type(world, ct->as_type, str);
+    if (flecs_json_typeinfo_ser_type_slice(
+        world, &op[1], op->op_count - 2, str)) 
+    {
+        return -1;
     }
 
-    flecs_json_array_push(str);
-
-    switch(op->kind) {
-    case EcsOpPush:
-    case EcsOpPop:
-        /* Should not be parsed as single op */
-        ecs_throw(ECS_INVALID_PARAMETER, 
-            "unexpected push/pop serializer instruction");
-        break;
-    case EcsOpEnum:
-        json_typeinfo_ser_enum(world, op->type, str);
-        break;
-    case EcsOpBitmask:
-        json_typeinfo_ser_bitmask(world, op->type, str);
-        break;
-    case EcsOpArray:
-        json_typeinfo_ser_array_type(world, op->type, str);
-        break;
-    case EcsOpVector:
-        json_typeinfo_ser_vector(world, op->type, str);
-        break;
-    case EcsOpOpaque:
-        /* Can't happen, already handled above */
-        ecs_throw(ECS_INTERNAL_ERROR, NULL);
-        break;
-    case EcsOpBool:
-    case EcsOpChar:
-    case EcsOpByte:
-    case EcsOpU8:
-    case EcsOpU16:
-    case EcsOpU32:
-    case EcsOpU64:
-    case EcsOpI8:
-    case EcsOpI16:
-    case EcsOpI32:
-    case EcsOpI64:
-    case EcsOpF32:
-    case EcsOpF64:
-    case EcsOpUPtr:
-    case EcsOpIPtr:
-    case EcsOpEntity:
-    case EcsOpId:
-    case EcsOpString:
-        if (json_typeinfo_ser_primitive( 
-            flecs_json_op_to_primitive_kind(op->kind), str))
-        {
-            ecs_throw(ECS_INTERNAL_ERROR, NULL);
-        }
-        break;
-    case EcsOpScope:
-    case EcsOpPrimitive:
-    default:
-        ecs_throw(ECS_INTERNAL_ERROR, NULL);
+    ecs_entity_t unit = ecs_get_target_for(world, op->type, EcsIsA, EcsUnit);
+    if (unit) {
+        flecs_json_member(str, "@self");
+        flecs_json_array_push(str);
+        flecs_json_object_push(str);
+        flecs_json_typeinfo_ser_unit(world, str, unit);
+        flecs_json_object_pop(str);
+        flecs_json_array_pop(str);
     }
 
-    if (st) {
-        ecs_member_t *m = ecs_vec_get_t(
-            &st->members, ecs_member_t, op->member_index);
-        ecs_assert(m != NULL, ECS_INTERNAL_ERROR, NULL);
+    return 0;
+}
 
-        bool value_range = ECS_NEQ(m->range.min, m->range.max);
-        bool error_range = ECS_NEQ(m->error_range.min, m->error_range.max);
-        bool warning_range = ECS_NEQ(m->warning_range.min, m->warning_range.max);
+static
+int flecs_json_typeinfo_ser_array(
+    const ecs_world_t *world,
+    const ecs_meta_op_t *op,
+    ecs_strbuf_t *str)
+{
+    ecs_strbuf_list_appendstr(str, "\"array\"");
 
-        ecs_entity_t unit = m->unit;
-        if (unit || error_range || warning_range || value_range) {
-            flecs_json_next(str);
-            flecs_json_next(str);
-            flecs_json_object_push(str);
-
-            if (unit) {
-                json_typeinfo_ser_unit(world, str, unit);
-            }
-            if (value_range) {
-                json_typeinfo_ser_range(str, "range", &m->range);
-            }
-            if (error_range) {
-                json_typeinfo_ser_range(str, "error_range", &m->error_range);
-            }
-            if (warning_range) {
-                json_typeinfo_ser_range(str, "warning_range", &m->warning_range);
-            }
-
-            flecs_json_object_pop(str);
-        }
+    flecs_json_next(str);
+    if (flecs_json_typeinfo_ser_scope(world, op, str)) {
+        goto error;
     }
 
-    flecs_json_array_pop(str);
+    ecs_strbuf_list_append(str, "%u", ecs_meta_op_get_elem_count(op, NULL));
+
+    return 0;
+error:
+    return -1;
+}
+
+static
+int flecs_json_typeinfo_ser_vector(
+    const ecs_world_t *world,
+    const ecs_meta_op_t *op,
+    ecs_strbuf_t *str)
+{
+    ecs_strbuf_list_appendstr(str, "\"vector\"");
+
+    flecs_json_next(str);
+    if (flecs_json_typeinfo_ser_scope(world, op, str)) {
+        goto error;
+    }
 
     return 0;
 error:
@@ -304,59 +207,73 @@ error:
 
 /* Iterate over a slice of the type ops array */
 static
-int json_typeinfo_ser_type_ops(
+int flecs_json_typeinfo_ser_type_slice(
     const ecs_world_t *world,
-    ecs_meta_type_op_t *ops,
+    const ecs_meta_op_t *ops,
     int32_t op_count,
-    ecs_strbuf_t *str,
-    const EcsStruct *st) 
+    ecs_strbuf_t *str) 
 {
-    const EcsStruct *stack[64] = {st};
-    int32_t sp = 1;
+    const EcsStruct *st = NULL;
+    if (ops[0].name) { /* If a member, previous operation is PushStruct */
+        const ecs_meta_op_t *push = &ops[-1];
+        ecs_assert(push->kind == EcsOpPushStruct, ECS_INTERNAL_ERROR, NULL);
+        st = ecs_get(world, push->type, EcsStruct);
+        ecs_assert(st != NULL, ECS_INTERNAL_ERROR, NULL);
+    }
 
     for (int i = 0; i < op_count; i ++) {
-        ecs_meta_type_op_t *op = &ops[i];
+        const ecs_meta_op_t *op = &ops[i];
 
-        if (op != ops) {
-            if (op->name) {
-                flecs_json_member(str, op->name);
-            }
+        if (op->name) {
+            flecs_json_member(str, op->name);
         }
 
-        int32_t elem_count = op->count;
-        if (elem_count > 1) {
-            flecs_json_array_push(str);
-            json_typeinfo_ser_array(world, op->type, op->count, str);
-            flecs_json_array_pop(str);
+        if (op->kind == EcsOpForward) {
+            if (flecs_json_typeinfo_ser_type(world, op->type, str)) {
+                goto error;
+            }
+            continue;
+        } else if (op->kind == EcsOpOpaqueStruct || 
+            op->kind == EcsOpOpaqueArray || op->kind == EcsOpOpaqueVector ||
+            op->kind == EcsOpOpaqueValue) 
+        {
+            const EcsOpaque *ct = ecs_get(world, op->type, EcsOpaque);
+            ecs_assert(ct != NULL, ECS_INTERNAL_ERROR, NULL);
+            if (flecs_json_typeinfo_ser_type(world, ct->as_type, str)) {
+                goto error;
+            }
+            continue;
+        } else if (op->kind == EcsOpPushStruct) {
+            flecs_json_object_push(str);
+            if (flecs_json_typeinfo_ser_scope(world, op, str)) {
+                return -1;
+            }
+            flecs_json_object_pop(str);
             i += op->op_count - 1;
             continue;
         }
 
-        switch(op->kind) {
-        case EcsOpPush:
-            flecs_json_object_push(str);
-            ecs_assert(sp < 63, ECS_INVALID_OPERATION, "type nesting too deep");
-            stack[sp ++] = ecs_get(world, op->type, EcsStruct);
-            break;
-        case EcsOpPop: {
-            ecs_entity_t unit = ecs_get_target_for(world, op->type, EcsIsA, EcsUnit);
-            if (unit) {
-                flecs_json_member(str, "@self");
-                flecs_json_array_push(str);
-                flecs_json_object_push(str);
-                json_typeinfo_ser_unit(world, str, unit);
-                flecs_json_object_pop(str);
-                flecs_json_array_pop(str);
-            }
+        flecs_json_array_push(str);
 
-            flecs_json_object_pop(str);
-            sp --;
+        switch(op->kind) {
+        case EcsOpPushArray:
+            if (flecs_json_typeinfo_ser_array(world, op, str)) {
+                goto error;
+            }
+            i += op->op_count - 1;
+            break;
+        case EcsOpPushVector:
+            if (flecs_json_typeinfo_ser_vector(world, op, str)) {
+                goto error;
+            }
+            i += op->op_count - 1;
             break;
-        }
-        case EcsOpArray:
-        case EcsOpVector:
         case EcsOpEnum:
+            flecs_json_typeinfo_ser_enum(world, op->type, str);
+            break;
         case EcsOpBitmask:
+            flecs_json_typeinfo_ser_bitmask(world, op->type, str);
+            break;
         case EcsOpBool:
         case EcsOpChar:
         case EcsOpByte:
@@ -375,16 +292,62 @@ int json_typeinfo_ser_type_ops(
         case EcsOpEntity:
         case EcsOpId:
         case EcsOpString:
-        case EcsOpOpaque:
-            if (json_typeinfo_ser_type_op(world, op, str, stack[sp - 1])) {
-                goto error;
+            if (flecs_json_typeinfo_ser_primitive( 
+                flecs_json_op_to_primitive_kind(op->kind), str))
+            {
+                ecs_throw(ECS_INTERNAL_ERROR, NULL);
             }
             break;
-        case EcsOpPrimitive:
         case EcsOpScope:
+        case EcsOpPrimitive:
+        case EcsOpPop:
+        case EcsOpForward:
+        case EcsOpOpaqueStruct:
+        case EcsOpOpaqueArray:
+        case EcsOpOpaqueVector:
+        case EcsOpOpaqueValue:
+        case EcsOpPushStruct:
         default:
             ecs_throw(ECS_INTERNAL_ERROR, NULL);
         }
+
+        if (op->name) {
+            ecs_assert(st != NULL, ECS_INTERNAL_ERROR, NULL);
+            ecs_member_t *m = ecs_vec_get_t(
+                &st->members, ecs_member_t, op->member_index);
+            ecs_assert(m != NULL, ECS_INTERNAL_ERROR, NULL);
+
+            bool value_range = ECS_NEQ(m->range.min, m->range.max);
+            bool error_range = ECS_NEQ(m->error_range.min, m->error_range.max);
+            bool warning_range = ECS_NEQ(
+                m->warning_range.min, m->warning_range.max);
+
+            ecs_entity_t unit = m->unit;
+            if (unit || error_range || warning_range || value_range) {
+                flecs_json_next(str);
+                flecs_json_next(str);
+                flecs_json_object_push(str);
+
+                if (unit) {
+                    flecs_json_typeinfo_ser_unit(world, str, unit);
+                }
+                if (value_range) {
+                    flecs_json_typeinfo_ser_range(str, "range", &m->range);
+                }
+                if (error_range) {
+                    flecs_json_typeinfo_ser_range(
+                        str, "error_range", &m->error_range);
+                }
+                if (warning_range) {
+                    flecs_json_typeinfo_ser_range(
+                        str, "warning_range", &m->warning_range);
+                }
+
+                flecs_json_object_pop(str);
+            }
+        }
+
+        flecs_json_array_pop(str);
     }
 
     return 0;
@@ -393,7 +356,7 @@ error:
 }
 
 static
-int json_typeinfo_ser_type(
+int flecs_json_typeinfo_ser_type(
     const ecs_world_t *world,
     ecs_entity_t type,
     ecs_strbuf_t *buf)
@@ -411,11 +374,10 @@ int json_typeinfo_ser_type(
         return 0;
     }
 
-    const EcsStruct *st = ecs_get(world, type, EcsStruct);
-    ecs_meta_type_op_t *ops = ecs_vec_first_t(&ser->ops, ecs_meta_type_op_t);
+    ecs_meta_op_t *ops = ecs_vec_first_t(&ser->ops, ecs_meta_op_t);
     int32_t count = ecs_vec_count(&ser->ops);
 
-    if (json_typeinfo_ser_type_ops(world, ops, count, buf, st)) {
+    if (flecs_json_typeinfo_ser_type_slice(world, ops, count, buf)) {
         return -1;
     }
 
@@ -427,7 +389,7 @@ int ecs_type_info_to_json_buf(
     ecs_entity_t type,
     ecs_strbuf_t *buf)
 {
-    return json_typeinfo_ser_type(world, type, buf);
+    return flecs_json_typeinfo_ser_type(world, type, buf);
 }
 
 char* ecs_type_info_to_json(
