@@ -38,7 +38,7 @@ void MonitorStats(ecs_iter_t *it) {
     ecs_world_t *world = it->real_world;
     ecs_monitor_stats_ctx_t *ctx = it->ctx;
 
-    EcsStatsHeader *hdr = ecs_field_w_size(it, 0, 0);
+    EcsStatsHeader *hdr = ecs_field_w_size(it, ecs_field_size(it, 0), 0);
 
     ecs_ftime_t elapsed = hdr->elapsed;
     hdr->elapsed += it->delta_time;
@@ -128,8 +128,8 @@ static
 void ReduceStats(ecs_iter_t *it) {
     ecs_reduce_stats_ctx_t *ctx = it->ctx;
 
-    void *dst = ecs_field_w_size(it, 0, 0);
-    void *src = ecs_field_w_size(it, 0, 1);
+    void *dst = ecs_field_w_size(it, ecs_field_size(it, 0), 0);
+    void *src = ecs_field_w_size(it, ecs_field_size(it, 1), 1);
 
     dst = ECS_OFFSET_T(dst, EcsStatsHeader);
     src = ECS_OFFSET_T(src, EcsStatsHeader);
@@ -152,8 +152,8 @@ void AggregateStats(ecs_iter_t *it) {
     ecs_aggregate_stats_ctx_t *ctx = it->ctx;
     int32_t interval = ctx->interval;
 
-    EcsStatsHeader *dst_hdr = ecs_field_w_size(it, 0, 0);
-    EcsStatsHeader *src_hdr = ecs_field_w_size(it, 0, 1);
+    EcsStatsHeader *dst_hdr = ecs_field_w_size(it, ecs_field_size(it, 0), 0);
+    EcsStatsHeader *src_hdr = ecs_field_w_size(it, ecs_field_size(it, 1), 1);
 
     void *dst = ECS_OFFSET_T(dst_hdr, EcsStatsHeader);
     void *src = ECS_OFFSET_T(src_hdr, EcsStatsHeader);
