@@ -13,7 +13,7 @@ void flecs_trav_entity_down_isa(
     ecs_trav_down_t *dst,
     ecs_entity_t trav,
     ecs_entity_t entity,
-    ecs_component_record_t *idr_with,
+    ecs_component_record_t *cr_with,
     bool self,
     bool empty);
 
@@ -24,8 +24,8 @@ ecs_trav_down_t* flecs_trav_entity_down(
     ecs_trav_up_cache_t *cache,
     ecs_trav_down_t *dst,
     ecs_entity_t trav,
-    ecs_component_record_t *idr_trav,
-    ecs_component_record_t *idr_with,
+    ecs_component_record_t *cr_trav,
+    ecs_component_record_t *cr_with,
     bool self,
     bool empty);
 
@@ -53,7 +53,7 @@ ecs_trav_down_t* flecs_trav_table_down(
     ecs_trav_down_t *dst,
     ecs_entity_t trav,
     const ecs_table_t *table,
-    ecs_component_record_t *idr_with,
+    ecs_component_record_t *cr_with,
     bool self,
     bool empty)
 {
@@ -63,7 +63,7 @@ ecs_trav_down_t* flecs_trav_table_down(
         return dst;
     }
 
-    ecs_assert(idr_with != NULL, ECS_INTERNAL_ERROR, NULL);
+    ecs_assert(cr_with != NULL, ECS_INTERNAL_ERROR, NULL);
 
     const ecs_entity_t *entities = ecs_table_entities(table);
     int32_t i, count = ecs_table_count(table);
@@ -76,14 +76,14 @@ ecs_trav_down_t* flecs_trav_table_down(
 
         uint32_t flags = ECS_RECORD_TO_ROW_FLAGS(record->row);
         if (flags & EcsEntityIsTraversable) {
-            ecs_component_record_t *idr_trav = flecs_components_get(world, 
+            ecs_component_record_t *cr_trav = flecs_components_get(world, 
                 ecs_pair(trav, entity));
-            if (!idr_trav) {
+            if (!cr_trav) {
                 continue;
             }
 
             flecs_trav_entity_down(world, a, cache, dst, 
-                trav, idr_trav, idr_with, self, empty);
+                trav, cr_trav, cr_with, self, empty);
         }
     }
 
@@ -98,22 +98,22 @@ void flecs_trav_entity_down_isa(
     ecs_trav_down_t *dst,
     ecs_entity_t trav,
     ecs_entity_t entity,
-    ecs_component_record_t *idr_with,
+    ecs_component_record_t *cr_with,
     bool self,
     bool empty)
 {
-    if (trav == EcsIsA || !world->idr_isa_wildcard) {
+    if (trav == EcsIsA || !world->cr_isa_wildcard) {
         return;
     }
 
-    ecs_component_record_t *idr_isa = flecs_components_get(
+    ecs_component_record_t *cr_isa = flecs_components_get(
         world, ecs_pair(EcsIsA, entity));
-    if (!idr_isa) {
+    if (!cr_isa) {
         return;
     }
 
     ecs_table_cache_iter_t it;
-    if (flecs_table_cache_iter(&idr_isa->cache, &it)) {
+    if (flecs_table_cache_iter(&cr_isa->cache, &it)) {
         const ecs_table_record_t *tr;
         while ((tr = flecs_table_cache_next(&it, ecs_table_record_t))) {
             ecs_table_t *table = tr->hdr.table;
@@ -121,7 +121,7 @@ void flecs_trav_entity_down_isa(
                 continue;
             }
 
-            if (ecs_table_has_id(world, table, idr_with->id)) {
+            if (ecs_table_has_id(world, table, cr_with->id)) {
                 /* Table owns component */
                 continue;
             }
@@ -137,15 +137,15 @@ void flecs_trav_entity_down_isa(
 
                 uint32_t flags = ECS_RECORD_TO_ROW_FLAGS(record->row);
                 if (flags & EcsEntityIsTraversable) {
-                    ecs_component_record_t *idr_trav = flecs_components_get(world, 
+                    ecs_component_record_t *cr_trav = flecs_components_get(world, 
                         ecs_pair(trav, e));
-                    if (idr_trav) {
+                    if (cr_trav) {
                         flecs_trav_entity_down(world, a, cache, dst, trav,
-                            idr_trav, idr_with, self, empty);
+                            cr_trav, cr_with, self, empty);
                     }
 
                     flecs_trav_entity_down_isa(world, a, cache, dst, trav, e, 
-                        idr_with, self, empty);
+                        cr_with, self, empty);
                 }
             }
         }
@@ -159,23 +159,23 @@ ecs_trav_down_t* flecs_trav_entity_down(
     ecs_trav_up_cache_t *cache,
     ecs_trav_down_t *dst,
     ecs_entity_t trav,
-    ecs_component_record_t *idr_trav,
-    ecs_component_record_t *idr_with,
+    ecs_component_record_t *cr_trav,
+    ecs_component_record_t *cr_with,
     bool self,
     bool empty)
 {
     ecs_assert(dst != NULL, ECS_INTERNAL_ERROR, NULL);
-    ecs_assert(idr_with != NULL, ECS_INTERNAL_ERROR, NULL);
-    ecs_assert(idr_trav != NULL, ECS_INTERNAL_ERROR, NULL);
+    ecs_assert(cr_with != NULL, ECS_INTERNAL_ERROR, NULL);
+    ecs_assert(cr_trav != NULL, ECS_INTERNAL_ERROR, NULL);
 
     int32_t first = ecs_vec_count(&dst->elems);
 
     ecs_table_cache_iter_t it;
     bool result;
     if (empty) {
-        result = flecs_table_cache_all_iter(&idr_trav->cache, &it);
+        result = flecs_table_cache_all_iter(&cr_trav->cache, &it);
     } else {
-        result = flecs_table_cache_iter(&idr_trav->cache, &it);
+        result = flecs_table_cache_iter(&cr_trav->cache, &it);
     }
 
     if (result) {
@@ -185,7 +185,7 @@ ecs_trav_down_t* flecs_trav_entity_down(
             ecs_table_t *table = tr->hdr.table;
             bool leaf = false;
 
-            if (flecs_component_get_table(idr_with, table) != NULL) {
+            if (flecs_component_get_table(cr_with, table) != NULL) {
                 if (self) {
                     continue;
                 }
@@ -199,7 +199,7 @@ ecs_trav_down_t* flecs_trav_entity_down(
                 ecs_id_t id = table->type.array[index - 1];
                 if (ECS_IS_PAIR(id) && ECS_PAIR_FIRST(id) == trav) {
                     int32_t col = ecs_search_relation(world, table, 0, 
-                        idr_with->id, trav, EcsUp, NULL, NULL, &tr);
+                        cr_with->id, trav, EcsUp, NULL, NULL, &tr);
                     ecs_assert(col >= 0, ECS_INTERNAL_ERROR, NULL);
 
                     if (col != index) {
@@ -224,7 +224,7 @@ ecs_trav_down_t* flecs_trav_entity_down(
             &dst->elems, ecs_trav_down_elem_t, t);
         if (!elem->leaf) {
             flecs_trav_table_down(world, a, cache, dst, trav,
-                elem->table, idr_with, self, empty);
+                elem->table, cr_with, self, empty);
         }
     }
 
@@ -236,7 +236,7 @@ ecs_trav_down_t* flecs_query_get_down_cache(
     ecs_trav_up_cache_t *cache,
     ecs_entity_t trav,
     ecs_entity_t e,
-    ecs_component_record_t *idr_with,
+    ecs_component_record_t *cr_with,
     bool self,
     bool empty)
 {
@@ -252,11 +252,11 @@ ecs_trav_down_t* flecs_query_get_down_cache(
         return result;
     }
 
-    ecs_component_record_t *idr_trav = flecs_components_get(world, ecs_pair(trav, e));
-    if (!idr_trav) {
+    ecs_component_record_t *cr_trav = flecs_components_get(world, ecs_pair(trav, e));
+    if (!cr_trav) {
         if (trav != EcsIsA) {
             flecs_trav_entity_down_isa(
-                world, a, cache, result, trav, e, idr_with, self, empty);
+                world, a, cache, result, trav, e, cr_with, self, empty);
         }
         result->ready = true;
         return result;
@@ -266,13 +266,13 @@ ecs_trav_down_t* flecs_query_get_down_cache(
 
     /* Cover IsA -> trav paths. If a parent inherits a component, then children
      * of that parent should find the component through up traversal. */
-    if (idr_with->flags & EcsIdOnInstantiateInherit) {
+    if (cr_with->flags & EcsIdOnInstantiateInherit) {
         flecs_trav_entity_down_isa(
-            world, a, cache, result, trav, e, idr_with, self, empty);
+            world, a, cache, result, trav, e, cr_with, self, empty);
     }
 
     flecs_trav_entity_down(
-        world, a, cache, result, trav, idr_trav, idr_with, self, empty);
+        world, a, cache, result, trav, cr_trav, cr_with, self, empty);
     result->ready = true;
 
     return result;
