@@ -916,7 +916,6 @@ void Meta_ser_deser_std_vector_std_string(void) {
     world.component<std::string>()
         .opaque(std_string_support);
 
-
     world.component<std::vector<std::string>>()
         .opaque(std_vector_support<std::string>);
 
@@ -1234,16 +1233,16 @@ void Meta_enum_w_bits(void) {
 	ecs.component<EnumWithBitsStruct>()
 		.member<EnumWithBits>("bits");
 
-	for (int i = 0; i < 30; i++)
+	for (int i = 0; i < 1; i++)
 	{
-		ecs.entity()
-			.child_of(ecs.entity())
+		ecs.entity("e")
+			.child_of(ecs.entity("p"))
 			.add<EnumWithBitsStruct>();
 	}
 
     auto q = ecs.query<EnumWithBitsStruct>();
     auto s = q.iter().to_json();
-    test_str(s.c_str(), "");
+    test_str(s.c_str(), "{\"results\":[{\"parent\":\"p\", \"name\":\"e\", \"fields\":{\"values\":[{\"bits\":\"BitAll\"}]}}]}");
 }
 
 void Meta_value_range(void) {
@@ -1471,8 +1470,8 @@ void Meta_out_of_order_member_declaration(void) {
     flecs::world ecs;
 
     auto c = ecs.component<Position>()
-        .member<float>("y", 1, offsetof(Position, y))
-        .member<float>("x", 1, offsetof(Position, x));
+        .member<float>("y", 0, offsetof(Position, y))
+        .member<float>("x", 0, offsetof(Position, x));
     test_assert(c != 0);
 
     const flecs::Component *ptr = c.try_get<flecs::Component>();
@@ -1554,3 +1553,51 @@ void Meta_query_to_json_w_default_desc(void) {
 
     test_str(q.to_json(&desc).c_str(), "{\"results\":[{\"name\":\"foo\", \"fields\":{\"values\":[0]}}]}");
 }
+
+void Meta_script_to_std_vector_int(void) {
+    flecs::world world;
+
+    world.component<std::vector<int>>("IntVec")
+        .opaque(std_vector_support<int>);
+
+    flecs::entity s = world.script()
+        .code("e { IntVec: [10, 20, 30] }")
+        .run();
+
+    const flecs::Script& sptr = s.get<flecs::Script>();
+    test_assert(sptr.error == nullptr);
+
+    flecs::entity e = world.lookup("e");
+    test_assert(e != 0);
+
+    const std::vector<int>& v = e.get<std::vector<int>>();
+    test_int(v.size(), 3);
+    test_int(v.at(0), 10);
+    test_int(v.at(1), 20);
+    test_int(v.at(2), 30);
+}
+
+void Meta_script_to_std_vector_std_string(void) {
+    flecs::world world;
+
+    world.component<std::string>()
+        .opaque(std_string_support);
+
+    world.component<std::vector<std::string>>("StringVec")
+        .opaque(std_vector_support<std::string>);
+
+    flecs::entity s = world.script()
+        .code("e { StringVec: [\"Hello\", \"World\"] }")
+        .run();
+
+    const flecs::Script& sptr = s.get<flecs::Script>();
+    test_assert(sptr.error == nullptr);
+
+    flecs::entity e = world.lookup("e");
+    test_assert(e != 0);
+
+    const std::vector<std::string>& v = e.get<std::vector<std::string>>();
+    test_int(v.size(), 2);
+    test_str(v.at(0).c_str(), "Hello");
+    test_str(v.at(1).c_str(), "World");
+}
