@@ -13,7 +13,7 @@ void Error_multi_line_comment_after_newline_before_newline_scope_open(void) {
     LINE "Foo{}";
 
     ecs_log_set_level(-4); /* Newline after multiline comment is not ignored */
-    test_assert(ecs_script_run(world, NULL, expr) != 0);
+    test_assert(ecs_script_run(world, NULL, expr, NULL) != 0);
 
     ecs_fini(world);
 }
@@ -26,7 +26,7 @@ void Error_missing_end_of_scope(void) {
     HEAD "Parent {"
     LINE " Child {}";
 
-    test_assert(ecs_script_run(world, NULL, expr) != 0);
+    test_assert(ecs_script_run(world, NULL, expr, NULL) != 0);
 
     test_assert(ecs_get_scope(world) == 0);
     test_assert(ecs_get_with(world) == 0);
@@ -57,7 +57,7 @@ void Error_with_n_tags_2_levels_invalid_tag(void) {
     LINE "HelloC {}";
 
     ecs_log_set_level(-4);
-    test_assert(ecs_script_run(world, NULL, expr) != 0);
+    test_assert(ecs_script_run(world, NULL, expr, NULL) != 0);
 
     ecs_fini(world);
 }
@@ -69,7 +69,7 @@ void Error_assignment_to_non_component(void) {
     HEAD "Foo { Position: {x: 10, y: 20} }";
 
     ecs_log_set_level(-4);
-    test_assert(ecs_script_run(world, NULL, expr) != 0);
+    test_assert(ecs_script_run(world, NULL, expr, NULL) != 0);
 
     ecs_fini(world);
 }
@@ -83,7 +83,7 @@ void Error_struct_w_member_w_assignment_to_nothing(void) {
     LINE "}";
 
     ecs_log_set_level(-4);
-    test_assert(ecs_script_run(world, NULL, expr) != 0);
+    test_assert(ecs_script_run(world, NULL, expr, NULL) != 0);
 
     ecs_fini(world);
 }
@@ -97,7 +97,7 @@ void Error_struct_w_member_w_assignment_to_empty_scope(void) {
     LINE "}";
 
     ecs_log_set_level(-4);
-    test_assert(ecs_script_run(world, NULL, expr) != 0);
+    test_assert(ecs_script_run(world, NULL, expr, NULL) != 0);
 
     ecs_fini(world);
 }
@@ -110,7 +110,7 @@ void Error_invalid_nested_assignment(void) {
     LINE "Bar { Hello }";
 
     ecs_log_set_level(-4);
-    test_assert(ecs_script_run(world, NULL, expr) != 0);
+    test_assert(ecs_script_run(world, NULL, expr, NULL) != 0);
 
     ecs_fini(world);
 }
@@ -122,7 +122,7 @@ void Error_invalid_partial_pair_assignment(void) {
     HEAD "Foo { (Hello, }";
 
     ecs_log_set_level(-4);
-    test_assert(ecs_script_run(world, NULL, expr) != 0);
+    test_assert(ecs_script_run(world, NULL, expr, NULL) != 0);
 
     ecs_fini(world);
 }
@@ -134,7 +134,7 @@ void Error_empty_assignment(void) {
     HEAD "Foo {";
 
     ecs_log_set_level(-4);
-    test_assert(ecs_script_run(world, NULL, expr) != 0);
+    test_assert(ecs_script_run(world, NULL, expr, NULL) != 0);
 
     ecs_fini(world);
 }
@@ -146,7 +146,7 @@ void Error_empty_assignment_before_end_of_scope(void) {
     HEAD "{Foo {}";
 
     ecs_log_set_level(-4);
-    test_assert(ecs_script_run(world, NULL, expr) != 0);
+    test_assert(ecs_script_run(world, NULL, expr, NULL) != 0);
 
     ecs_fini(world);
 }
@@ -161,7 +161,7 @@ void Error_default_type_with_tag(void) {
     LINE "}";
 
     ecs_log_set_level(-4);
-    test_assert(ecs_script_run(world, NULL, expr) != 0);
+    test_assert(ecs_script_run(world, NULL, expr, NULL) != 0);
 
     ecs_fini(world);
 }
@@ -179,7 +179,7 @@ void Error_invalid_oneof(void) {
     LINE "}"
     LINE "e { (Color, Foo) }";
 
-    test_assert(ecs_script_run(world, NULL, expr) != 0);
+    test_assert(ecs_script_run(world, NULL, expr, NULL) != 0);
 
     ecs_entity_t color = ecs_lookup(world, "Color");
     ecs_entity_t foo = ecs_lookup(world, "Foo");
@@ -212,7 +212,7 @@ void Error_unterminated_multiline_string(void) {
     LINE "}}";
 
     ecs_log_set_level(-4);
-    test_assert(ecs_script_run(world, NULL, expr) != 0);
+    test_assert(ecs_script_run(world, NULL, expr, NULL) != 0);
 
     ecs_fini(world);
 }
@@ -231,7 +231,7 @@ void Error_invalid_assign_multiline_string(void) {
     HEAD "Foo { String: {value: `foo`} }";
 
     ecs_log_set_level(-4);
-    test_assert(ecs_script_run(world, NULL, expr) != 0);
+    test_assert(ecs_script_run(world, NULL, expr, NULL) != 0);
 
     ecs_fini(world);
 }
@@ -244,7 +244,7 @@ void Error_const_var_redeclare(void) {
     LINE "const var_x = 20";
 
     ecs_log_set_level(-4);
-    test_assert(ecs_script_run(world, NULL, expr) != 0);
+    test_assert(ecs_script_run(world, NULL, expr, NULL) != 0);
 
     ecs_fini(world);
 }
@@ -266,7 +266,7 @@ void Error_typed_const_w_composite_type_invalid_assignment(void) {
     LINE "";
 
     ecs_log_set_level(-4);
-    test_assert(ecs_script_run(world, NULL, expr) != 0);
+    test_assert(ecs_script_run(world, NULL, expr, NULL) != 0);
 
     ecs_fini(world);
 }
@@ -292,7 +292,7 @@ void Error_unterminated_multi_line_comment_in_value(void) {
     LINE "  y: 20\n"
     LINE "}}";
     ecs_log_set_level(-4);
-    test_assert(ecs_script_run(world, NULL, expr) != 0);
+    test_assert(ecs_script_run(world, NULL, expr, NULL) != 0);
 
     ecs_fini(world);
 }
@@ -311,7 +311,7 @@ void Error_pair_w_rel_var_invalid_type(void) {
     LINE "\n";
 
     ecs_log_set_level(-4);
-    test_assert(ecs_script_run(world, NULL, expr) != 0);
+    test_assert(ecs_script_run(world, NULL, expr, NULL) != 0);
 
     ecs_fini(world);
 }
@@ -330,7 +330,7 @@ void Error_pair_w_tgt_var_invalid_type(void) {
     LINE "\n";
 
     ecs_log_set_level(-4);
-    test_assert(ecs_script_run(world, NULL, expr) != 0);
+    test_assert(ecs_script_run(world, NULL, expr, NULL) != 0);
 
     ecs_fini(world);
 }
@@ -345,7 +345,7 @@ void Error_with_value_not_a_component(void) {
     LINE "\n";
 
     ecs_log_set_level(-4);
-    test_assert(ecs_script_run(world, NULL, expr) != 0);
+    test_assert(ecs_script_run(world, NULL, expr, NULL) != 0);
 
     ecs_fini(world);
 }
@@ -370,7 +370,7 @@ void Error_tag_in_with_scope(void) {
     LINE "\n";
 
     ecs_log_set_level(-4);
-    test_assert(ecs_script_run(world, NULL, expr) != 0);
+    test_assert(ecs_script_run(world, NULL, expr, NULL) != 0);
 
     ecs_fini(world);
 }
@@ -391,7 +391,7 @@ void Error_tag_in_with_scope_2(void) {
     ;
 
     ecs_log_set_level(-4);
-    test_assert(ecs_script_run(world, NULL, expr) != 0);
+    test_assert(ecs_script_run(world, NULL, expr, NULL) != 0);
 
     ecs_fini(world);
 }
@@ -413,7 +413,7 @@ void Error_pair_tag_in_with_scope_2(void) {
     ;
 
     ecs_log_set_level(-4);
-    test_assert(ecs_script_run(world, NULL, expr) != 0);
+    test_assert(ecs_script_run(world, NULL, expr, NULL) != 0);
 
     ecs_fini(world);
 }
@@ -438,7 +438,7 @@ void Error_component_in_with_scope(void) {
     LINE "\n";
 
     ecs_log_set_level(-4);
-    test_assert(ecs_script_run(world, NULL, expr) != 0);
+    test_assert(ecs_script_run(world, NULL, expr, NULL) != 0);
 
     ecs_fini(world);
 }
@@ -482,7 +482,7 @@ void Error_component_in_with_scope_2(void) {
     ;
 
     ecs_log_set_level(-4);
-    test_assert(ecs_script_run(world, NULL, expr) != 0);
+    test_assert(ecs_script_run(world, NULL, expr, NULL) != 0);
 
     ecs_fini(world);
 }
@@ -514,7 +514,7 @@ void Error_component_in_with_scope_3(void) {
     ;
 
     ecs_log_set_level(-4);
-    test_assert(ecs_script_run(world, NULL, expr) != 0);
+    test_assert(ecs_script_run(world, NULL, expr, NULL) != 0);
 
     ecs_fini(world);
 }
@@ -548,7 +548,7 @@ void Error_component_in_with_scope_4(void) {
     ;
 
     ecs_log_set_level(-4);
-    test_assert(ecs_script_run(world, NULL, expr) != 0);
+    test_assert(ecs_script_run(world, NULL, expr, NULL) != 0);
 
     ecs_fini(world);
 }
@@ -583,7 +583,7 @@ void Error_component_in_with_scope_5(void) {
     ;
 
     ecs_log_set_level(-4);
-    test_assert(ecs_script_run(world, NULL, expr) != 0);
+    test_assert(ecs_script_run(world, NULL, expr, NULL) != 0);
 
     ecs_fini(world);
 }
@@ -619,7 +619,7 @@ void Error_component_in_with_in_template(void) {
     ;
 
     ecs_log_set_level(-4);
-    test_assert(ecs_script_run(world, NULL, expr) != 0);
+    test_assert(ecs_script_run(world, NULL, expr, NULL) != 0);
 
     ecs_fini(world);
 }
@@ -655,7 +655,7 @@ void Error_component_in_with_scope_nested(void) {
     LINE "\n";
 
     ecs_log_set_level(-4);
-    test_assert(ecs_script_run(world, NULL, expr) != 0);
+    test_assert(ecs_script_run(world, NULL, expr, NULL) != 0);
 
     ecs_fini(world);
 }
@@ -681,7 +681,7 @@ void Error_component_in_with_scope_after_entity(void) {
     LINE "";
 
     ecs_log_set_level(-4);
-    test_assert(ecs_script_run(world, NULL, expr) != 0);
+    test_assert(ecs_script_run(world, NULL, expr, NULL) != 0);
 
     ecs_fini(world);
 }
@@ -707,7 +707,7 @@ void Error_component_in_with_var_scope(void) {
     LINE "";
 
     ecs_log_set_level(-4);
-    test_assert(ecs_script_run(world, NULL, expr) != 0);
+    test_assert(ecs_script_run(world, NULL, expr, NULL) != 0);
 
     ecs_fini(world);
 }
@@ -743,7 +743,7 @@ void Error_assign_after_with_in_scope(void) {
     LINE "}\n";
 
     ecs_log_set_level(-4);
-    test_assert(ecs_script_run(world, NULL, expr) == 0);
+    test_assert(ecs_script_run(world, NULL, expr, NULL) == 0);
 
     ecs_entity_t foo = ecs_lookup(world, "foo");
     test_assert(foo != 0);
@@ -781,7 +781,7 @@ void Error_not_an_array_component(void) {
     LINE "foo { Position: [10, 20] }\n";
 
     ecs_log_set_level(-4);
-    test_assert(ecs_script_run(world, NULL, expr) != 0);
+    test_assert(ecs_script_run(world, NULL, expr, NULL) != 0);
 
     ecs_fini(world);
 }
@@ -801,7 +801,7 @@ void Error_array_component_w_curly_brackets(void) {
     LINE "foo { Position: {10, 20} }\n";
 
     ecs_log_set_level(-4);
-    test_assert(ecs_script_run(world, NULL, expr) != 0);
+    test_assert(ecs_script_run(world, NULL, expr, NULL) != 0);
 
     ecs_fini(world);
 }
@@ -819,7 +819,7 @@ void Error_unknown_identifier(void) {
     LINE "Foo { Comp: {A} }";
 
     ecs_log_set_level(-4);
-    test_assert(ecs_script_run(world, NULL, expr) != 0);
+    test_assert(ecs_script_run(world, NULL, expr, NULL) != 0);
 
     ecs_fini(world);
 }
@@ -837,7 +837,7 @@ void Error_unknown_identifier_for_int_field(void) {
     LINE "Foo { Comp: {A} }";
 
     ecs_log_set_level(-4);
-    test_assert(ecs_script_run(world, NULL, expr) != 0);
+    test_assert(ecs_script_run(world, NULL, expr, NULL) != 0);
 
     ecs_fini(world);
 }
@@ -849,7 +849,7 @@ void Error_prefab_w_slot_no_parent(void) {
     HEAD "slot Base {}";
 
     ecs_log_set_level(-4);
-    test_assert(ecs_script_run(world, NULL, expr) != 0);
+    test_assert(ecs_script_run(world, NULL, expr, NULL) != 0);
 
     ecs_fini(world);
 }
@@ -863,7 +863,7 @@ void Error_tag_not_found(void) {
     LINE "}";
 
     ecs_log_set_level(-4);
-    test_assert(ecs_script_run(world, NULL, expr) != 0);
+    test_assert(ecs_script_run(world, NULL, expr, NULL) != 0);
 
     ecs_fini(world);
 }
@@ -877,7 +877,7 @@ void Error_component_not_found(void) {
     LINE "}";
 
     ecs_log_set_level(-4);
-    test_assert(ecs_script_run(world, NULL, expr) != 0);
+    test_assert(ecs_script_run(world, NULL, expr, NULL) != 0);
 
     ecs_fini(world);
 }
@@ -893,7 +893,7 @@ void Error_pair_first_not_found(void) {
     LINE "}";
 
     ecs_log_set_level(-4);
-    test_assert(ecs_script_run(world, NULL, expr) != 0);
+    test_assert(ecs_script_run(world, NULL, expr, NULL) != 0);
 
     ecs_fini(world);
 }
@@ -909,7 +909,7 @@ void Error_pair_second_not_found(void) {
     LINE "}";
 
     ecs_log_set_level(-4);
-    test_assert(ecs_script_run(world, NULL, expr) != 0);
+    test_assert(ecs_script_run(world, NULL, expr, NULL) != 0);
 
     ecs_fini(world);
 }
@@ -923,7 +923,7 @@ void Error_kind_not_found(void) {
     HEAD "Foo Bar";
 
     ecs_log_set_level(-4);
-    test_assert(ecs_script_run(world, NULL, expr) != 0);
+    test_assert(ecs_script_run(world, NULL, expr, NULL) != 0);
 
     ecs_fini(world);
 }
@@ -937,7 +937,7 @@ void Error_base_not_found(void) {
     HEAD "Foo : Bar";
 
     ecs_log_set_level(-4);
-    test_assert(ecs_script_run(world, NULL, expr) != 0);
+    test_assert(ecs_script_run(world, NULL, expr, NULL) != 0);
 
     ecs_fini(world);
 }
@@ -955,7 +955,11 @@ void Error_entity_w_anonymous_tag(void) {
         .entity = ecs_entity(world, { .name = "main" }),
         .code = expr
     });
-    test_assert(s == 0);
+    test_assert(s != 0);
+
+    const EcsScript *script = ecs_get(world, s, EcsScript);
+    test_assert(script != NULL);
+    test_assert(script->error != NULL);
 
     ecs_fini(world);
 }
@@ -974,7 +978,7 @@ void Error_member_expr_without_value_end_of_scope(void) {
     });
 
     ecs_log_set_level(-4);
-    test_assert(ecs_script_run(world, NULL, "Position(x:)") != 0);
+    test_assert(ecs_script_run(world, NULL, "Position(x:)", NULL) != 0);
 
     ecs_fini(world);
 }
@@ -993,7 +997,7 @@ void Error_member_expr_without_value_comma(void) {
     });
 
     ecs_log_set_level(-4);
-    test_assert(ecs_script_run(world, NULL, "Position(x:,0)") != 0);
+    test_assert(ecs_script_run(world, NULL, "Position(x:,0)", NULL) != 0);
 
     ecs_fini(world);
 }
@@ -1012,7 +1016,7 @@ void Error_member_expr_without_value_newline(void) {
     });
 
     ecs_log_set_level(-4);
-    test_assert(ecs_script_run(world, NULL, "Position(x:\n)") != 0);
+    test_assert(ecs_script_run(world, NULL, "Position(x:\n)", NULL) != 0);
 
     ecs_fini(world);
 }
@@ -1031,7 +1035,7 @@ void Error_2_member_expr_without_value(void) {
     });
 
     ecs_log_set_level(-4);
-    test_assert(ecs_script_run(world, NULL, "Position(x:y:)") != 0);
+    test_assert(ecs_script_run(world, NULL, "Position(x:y:)", NULL) != 0);
 
     ecs_fini(world);
 }
@@ -1050,7 +1054,7 @@ void Error_expr_junk_after_number(void) {
     });
 
     ecs_log_set_level(-4);
-    test_assert(ecs_script_run(world, NULL, "Position(0abc)") != 0);
+    test_assert(ecs_script_run(world, NULL, "Position(0abc)", NULL) != 0);
 
     ecs_fini(world);
 }
@@ -1069,7 +1073,7 @@ void Error_expr_junk_after_unary_minus(void) {
     });
 
     ecs_log_set_level(-4);
-    test_assert(ecs_script_run(world, NULL, "Position(-abc)") != 0);
+    test_assert(ecs_script_run(world, NULL, "Position(-abc)", NULL) != 0);
 
     ecs_fini(world);
 }
@@ -1088,7 +1092,7 @@ void Error_expr_comma_after_nothing(void) {
     });
 
     ecs_log_set_level(-4);
-    test_assert(ecs_script_run(world, NULL, "Position(,)") != 0);
+    test_assert(ecs_script_run(world, NULL, "Position(,)", NULL) != 0);
 
     ecs_fini(world);
 }
@@ -1107,7 +1111,7 @@ void Error_expr_digit_with_two_dots(void) {
     });
 
     ecs_log_set_level(-4);
-    test_assert(ecs_script_run(world, NULL, "Position(10.10.10)") != 0);
+    test_assert(ecs_script_run(world, NULL, "Position(10.10.10)", NULL) != 0);
 
     ecs_fini(world);
 }
@@ -1120,7 +1124,7 @@ void Error_template_empty(void) {
     LINE "}";
 
     ecs_log_set_level(-4);
-    test_assert(ecs_script_run(world, NULL, expr) == 0);
+    test_assert(ecs_script_run(world, NULL, expr, NULL) == 0);
 
     ecs_entity_t tree = ecs_lookup(world, "Tree");
     test_assert(tree != 0);
@@ -1138,7 +1142,7 @@ void Error_template_unresolved_tag(void) {
     LINE "Tree ent()";
 
     ecs_log_set_level(-4);
-    test_assert(ecs_script_run(world, NULL, expr) != 0);
+    test_assert(ecs_script_run(world, NULL, expr, NULL) != 0);
 
     ecs_fini(world);
 }
@@ -1157,7 +1161,7 @@ void Error_template_unresolved_component(void) {
     LINE "}";
 
     ecs_log_set_level(-4);
-    test_assert(ecs_script_run(world, NULL, expr) != 0);
+    test_assert(ecs_script_run(world, NULL, expr, NULL) != 0);
 
     ecs_fini(world);
 }
@@ -1173,7 +1177,7 @@ void Error_template_unresolved_pair_relationship(void) {
     LINE "Tree ent()";
 
     ecs_log_set_level(-4);
-    test_assert(ecs_script_run(world, NULL, expr) != 0);
+    test_assert(ecs_script_run(world, NULL, expr, NULL) != 0);
 
     ecs_fini(world);
 }
@@ -1189,7 +1193,7 @@ void Error_template_unresolved_pair_target(void) {
     LINE "Tree ent()";
 
     ecs_log_set_level(-4);
-    test_assert(ecs_script_run(world, NULL, expr) != 0);
+    test_assert(ecs_script_run(world, NULL, expr, NULL) != 0);
 
     ecs_fini(world);
 }
@@ -1205,7 +1209,7 @@ void Error_template_unresolved_with_tag(void) {
     LINE "Tree ent()";
 
     ecs_log_set_level(-4);
-    test_assert(ecs_script_run(world, NULL, expr) != 0);
+    test_assert(ecs_script_run(world, NULL, expr, NULL) != 0);
 
     ecs_fini(world);
 }
@@ -1225,7 +1229,7 @@ void Error_template_unresolved_with_component(void) {
     LINE "}";
 
     ecs_log_set_level(-4);
-    test_assert(ecs_script_run(world, NULL, expr) != 0);
+    test_assert(ecs_script_run(world, NULL, expr, NULL) != 0);
 
     ecs_fini(world);
 }
@@ -1242,7 +1246,7 @@ void Error_template_unresolved_with_pair_relationship(void) {
     LINE "Tree ent()";
 
     ecs_log_set_level(-4);
-    test_assert(ecs_script_run(world, NULL, expr) != 0);
+    test_assert(ecs_script_run(world, NULL, expr, NULL) != 0);
 
     ecs_fini(world);
 }
@@ -1259,7 +1263,7 @@ void Error_template_unresolved_with_pair_target(void) {
     LINE "Tree ent()";
 
     ecs_log_set_level(-4);
-    test_assert(ecs_script_run(world, NULL, expr) != 0);
+    test_assert(ecs_script_run(world, NULL, expr, NULL) != 0);
 
     ecs_fini(world);
 }
@@ -1276,7 +1280,7 @@ void Error_template_unresolved_tag_in_child(void) {
     LINE "Tree ent()";
 
     ecs_log_set_level(-4);
-    test_assert(ecs_script_run(world, NULL, expr) != 0);
+    test_assert(ecs_script_run(world, NULL, expr, NULL) != 0);
 
     ecs_fini(world);
 }
@@ -1300,7 +1304,7 @@ void Error_template_prop_no_type(void) {
     LINE "}";
 
     ecs_log_set_level(-4);
-    test_assert(ecs_script_run(world, NULL, expr) != 0);
+    test_assert(ecs_script_run(world, NULL, expr, NULL) != 0);
 
     ecs_fini(world);
 }
@@ -1327,7 +1331,7 @@ void Error_template_w_composite_prop_invalid_assignment(void) {
     LINE "t { Tree: {pos: {20, 30}} }";
 
     ecs_log_set_level(-4);
-    test_assert(ecs_script_run(world, NULL, expr) != 0);
+    test_assert(ecs_script_run(world, NULL, expr, NULL) != 0);
 
     ecs_fini(world);
 }
@@ -1345,7 +1349,7 @@ void Error_template_redeclare_prop_as_const(void) {
     LINE "\n";
 
     ecs_log_set_level(-4);
-    test_assert(ecs_script_run(world, NULL, expr) != 0);
+    test_assert(ecs_script_run(world, NULL, expr, NULL) != 0);
 
     ecs_fini(world);
 }
@@ -1363,7 +1367,7 @@ void Error_template_redeclare_prop_as_prop(void) {
     LINE "\n";
 
     ecs_log_set_level(-4);
-    test_assert(ecs_script_run(world, NULL, expr) != 0);
+    test_assert(ecs_script_run(world, NULL, expr, NULL) != 0);
 
     ecs_fini(world);
 }
@@ -1381,7 +1385,7 @@ void Error_template_redeclare_const_as_const(void) {
     LINE "\n";
 
     ecs_log_set_level(-4);
-    test_assert(ecs_script_run(world, NULL, expr) != 0);
+    test_assert(ecs_script_run(world, NULL, expr, NULL) != 0);
 
     ecs_fini(world);
 }
@@ -1401,7 +1405,7 @@ void Error_run_template_after_error(void) {
     LINE "  Fo"
     LINE "}";
 
-    test_assert(ecs_script_run(world, NULL, expr) == 0);
+    test_assert(ecs_script_run(world, NULL, expr, NULL) == 0);
 
     {
         ecs_entity_t tree = ecs_lookup(world, "Tree");
@@ -1412,7 +1416,7 @@ void Error_run_template_after_error(void) {
     }
 
     ecs_log_set_level(-4);
-    test_assert(ecs_script_run(world, NULL, expr_err) != 0);
+    test_assert(ecs_script_run(world, NULL, expr_err, NULL) != 0);
     ecs_log_set_level(-1);
 
     /* Because script is not managed, entiites created by script aren't deleted */
@@ -1425,7 +1429,7 @@ void Error_run_template_after_error(void) {
         test_assert(foo != 0);
     }
 
-    test_assert(ecs_script_run(world, NULL, expr) == 0);
+    test_assert(ecs_script_run(world, NULL, expr, NULL) == 0);
 
     {
         ecs_entity_t tree = ecs_lookup(world, "Tree");
@@ -1499,7 +1503,7 @@ void Error_template_in_template(void) {
     LINE "}";
 
     ecs_log_set_level(-4);
-    test_assert(ecs_script_run(world, NULL, expr) != 0);
+    test_assert(ecs_script_run(world, NULL, expr, NULL) != 0);
 
     ecs_fini(world);
 }
@@ -1620,7 +1624,7 @@ void Error_template_w_invalid_var_in_expr(void) {
     LINE "}";
 
     ecs_log_set_level(-4);
-    test_assert(ecs_script_run(world, NULL, expr) != 0);
+    test_assert(ecs_script_run(world, NULL, expr, NULL) != 0);
 
     ecs_fini(world);
 }
@@ -1669,7 +1673,252 @@ void Error_script_initializer_w_int_to_struct(void) {
     LINE "}";
 
     ecs_log_set_level(-4);
-    test_assert(ecs_script_run(world, NULL, expr) != 0);
+    test_assert(ecs_script_run(world, NULL, expr, NULL) != 0);
+
+    ecs_fini(world);
+}
+
+void Error_capture_error(void) {
+    ecs_world_t *world = ecs_init();
+
+    const char *expr =
+    HEAD "Parent {"
+    LINE " Child {}";
+
+    ecs_script_eval_result_t result = {0};
+    ecs_script_t *script = ecs_script_parse(world, "foo", expr, NULL, &result);
+    test_assert(script == NULL);
+    test_assert(result.error != NULL);
+    ecs_os_free(result.error);
+
+    ecs_fini(world);
+}
+
+void Error_unresolved_component_error_w_script_run(void) {
+    ecs_world_t *world = ecs_init();
+
+    const char *expr =
+    HEAD "e {"
+    LINE " Foo: {}"
+    LINE "}";
+
+    ecs_log_set_level(-4);
+
+    test_assert(ecs_script_run(world, NULL, expr, NULL) != 0);
+
+    ecs_fini(world);
+}
+
+void Error_unresolved_component_error_w_script_init(void) {
+    ecs_world_t *world = ecs_init();
+
+    const char *expr =
+    HEAD "e {"
+    LINE " Foo: {}"
+    LINE "}";
+
+    ecs_entity_t s = ecs_script(world, {
+        .code = expr
+    });
+
+    test_assert(s != 0);
+
+    const EcsScript *script = ecs_get(world, s, EcsScript);
+    test_assert(script != NULL);
+    test_assert(script->error != NULL);
+
+    ecs_fini(world);
+}
+
+void Error_unresolved_component_error_w_script_init_existing(void) {
+    ecs_world_t *world = ecs_init();
+
+    const char *expr =
+    HEAD "e {"
+    LINE " Foo: {}"
+    LINE "}";
+
+    ecs_entity_t s = ecs_script(world, {
+        .code = ""
+    });
+
+    test_assert(s != 0);
+
+    ecs_entity_t r = ecs_script(world, {
+        .entity = s,
+        .code = expr
+    });
+
+    test_assert(r != 0);
+    test_assert(r == s);
+
+    const EcsScript *script = ecs_get(world, s, EcsScript);
+    test_assert(script != NULL);
+    test_assert(script->script == NULL);
+    test_assert(script->error != NULL);
+
+    ecs_fini(world);
+}
+
+void Error_unresolved_component_error_w_script_eval(void) {
+    ecs_world_t *world = ecs_init();
+
+    const char *expr =
+    HEAD "e {"
+    LINE " Foo: {}"
+    LINE "}";
+
+    ecs_script_eval_result_t result = {0};
+    ecs_script_t *script = ecs_script_parse(world, "foo", expr, NULL, &result);
+
+    test_assert(script != NULL);
+    test_assert(result.error == NULL);
+
+    int r = ecs_script_eval(script, NULL, &result);
+    test_assert(r != 0);
+    test_assert(result.error != NULL);
+    ecs_os_free(result.error);
+
+    ecs_script_free(script);
+
+    ecs_fini(world);
+}
+
+void Error_unresolved_component_error_w_script_eval_multiple_times(void) {
+    ecs_world_t *world = ecs_init();
+
+    const char *expr =
+    HEAD "e {"
+    LINE " Foo: {}"
+    LINE "}";
+
+    ecs_script_eval_result_t result = {0};
+    ecs_script_t *script = ecs_script_parse(world, "foo", expr, NULL, &result);
+    test_assert(script != NULL);
+    test_assert(result.error == NULL);
+
+    int r = ecs_script_eval(script, NULL, &result);
+    test_assert(r != 0);
+    test_assert(result.error != NULL);
+
+    ecs_os_free(result.error);
+
+    r = ecs_script_eval(script, NULL, &result);
+    test_assert(r != 0);
+    test_assert(result.error != NULL);
+
+    ecs_os_free(result.error);
+    ecs_script_free(script);
+
+    ecs_fini(world);
+}
+
+void Error_annotation_without_newline(void) {
+    ecs_world_t *world = ecs_init();
+
+    const char *expr =
+    HEAD "@name Some annotation"
+    ;
+
+    ecs_log_set_level(-4);
+    test_assert(ecs_script_run(world, "foo", expr, NULL) != 0);
+
+    ecs_fini(world);
+}
+
+void Error_annotation_without_entity(void) {
+    ecs_world_t *world = ecs_init();
+
+    const char *expr =
+    HEAD "@name Some annotation"
+    LINE ""
+    ;
+
+    ecs_log_set_level(-4);
+    test_assert(ecs_script_run(world, "foo", expr, NULL) != 0);
+
+    ecs_fini(world);
+}
+
+
+void Error_annotation_to_unresolved_identifier(void) {
+    ecs_world_t *world = ecs_init();
+
+    const char *expr =
+    HEAD "@name Some annotation"
+    LINE "Foo"
+    ;
+
+    ecs_log_set_level(-4);
+    test_assert(ecs_script_run(world, "foo", expr, NULL) != 0);
+
+    ecs_fini(world);
+}
+
+void Error_annotation_to_unresolved_identifier_managed_parse_twice(void) {
+    ecs_world_t *world = ecs_init();
+
+    const char *expr =
+    HEAD "@name Some annotation"
+    LINE "Foo"
+    ;
+
+    ecs_entity_t s = ecs_script(world, {
+        .code = expr
+    });
+    test_assert(s != 0);
+
+    test_assert(ecs_script_update(world, s, 0, expr) != 0);
+
+    ecs_fini(world);
+}
+
+void Error_annotation_to_unresolved_identifier_managed_parse_twice_2(void) {
+    ecs_world_t *world = ecs_init();
+
+    const char *expr =
+    HEAD ""
+    LINE "notes {"
+    LINE "  @abcde"
+    LINE "  Note"
+    LINE "}"
+    LINE ""
+    LINE ""
+    ;
+
+    ecs_entity_t s = ecs_new(world);
+
+    ecs_script(world, {
+        .entity = s,
+        .code = expr
+    });
+    test_assert(s != 0);
+
+    ecs_script(world, {
+        .entity = s,
+        .code = expr
+    });
+
+    test_assert(s != 0);
+
+    ecs_fini(world);
+}
+
+void Error_annotation_to_tag(void) {
+    ecs_world_t *world = ecs_init();
+
+    const char *expr =
+    HEAD "Note {}"
+    LINE "notes {"
+    LINE "  @abcde"
+    LINE "  Note"
+    LINE "}"
+    LINE ""
+    LINE ""
+    ;
+
+    ecs_log_set_level(-4);
+    test_assert(ecs_script_run(world, "foo", expr, NULL) != 0);
 
     ecs_fini(world);
 }
